/**
 *  '$RCSfile: FileSystemDataStore.java,v $'
 *  Copyright: 2000 Regents of the University of California and the
 *              National Center for Ecological Analysis and Synthesis
 *    Authors: @authors@
 *    Release: @release@
 *
 *   '$Author: higgins $'
 *     '$Date: 2002/03/29 18:40:52 $'
 * '$Revision: 1.29 $'
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package edu.ucsb.nceas.morpho.framework;

import java.io.*;
import java.util.*;
import edu.ucsb.nceas.morpho.datapackage.*;

/**
 * implements and the DataStoreInterface for accessing files on the local
 * file system.
 */
public class FileSystemDataStore extends DataStore
                                 implements DataStoreInterface
{
  /**
   * create a new FileSystemDataStore for a ClientFramework
   */
  public FileSystemDataStore(ClientFramework cf)
  {
    super(cf);
  }
  
  /**
   * opens a file with the given name.  the name should be in the form
   * scope.accnum where the scope is unique to this machine.  The file will
   * be opened from the &lt;datadir&gt;/&lt;scope&gt;/ directory 
   * where the filename is the accnum.
   * Example: 
   *    name=johnson2343.13223
   *    datadir=data
   *    complete path=/usr/local/morpho/profiles/johnson/data/johnson2343/13223
   * Any characters after the first separator are assumed to be part of the 
   * accession number.  Hence the id johnson2343.13223.5 would produce 
   * the file johnson2343/13223.5
   */
  public File openFile(String name) throws FileNotFoundException
  {
    //debug(11, "opening files from: " + datadir);
    String path = parseId(name);
    path = datadir + "/" + path;
    File file = new File(path);
    if(!file.exists())
    {
      throw new FileNotFoundException("file " + path + " does not exist");
    }
    
    return file;
  }
  
  public File saveFile(String name, Reader file)
  {
    return saveFile(name, file, datadir, null);
  }
  
  public File saveFile(String name, Reader file, DataPackage dp)
  {
    return saveFile(name, file, datadir, dp);
  }
  
  public File saveTempFile(String name, Reader file)
  {
    return saveFile(name, file, tempdir, null);
  }

  public File saveDataFile(String name, Reader file)
  {
    return saveDataFile(name, file, datadir, null);
  }
  
  public File saveTempDataFile(String name, Reader file)
  {
    return saveDataFile(name, file, tempdir, null);
  }

  public File saveDataFile(String name, InputStream file)
  {
    return saveDataFile(name, file, datadir, null);
  }
  
  public File saveTempDataFile(String name, InputStream file)
  {
    return saveDataFile(name, file, tempdir, null);
  }
  
  /**
   * Saves a file with the given name.  if the file does not exist it is created
   * The file is saved according to the name provided.   The file will
   * be saved to the &lt;datadir&gt;/&lt;scope&gt;/ directory 
   * where the filename is the accnum.
   * Example: 
   *    name=johnson2343.13223
   *    datadir=data
   *    complete path=/usr/local/morpho/profiles/johnson/data/johnson2343/13223
   * Any characters after the first separator are assumed to be part of the 
   * accession number.  Hence the id johnson2343.13223.5 would produce 
   * the file johnson2343/13223.5
   */
  public File saveFile(String name, Reader file, String rootDir, DataPackage dp)
  {
    try
    {
      String path = parseId(name);
      String dirs = path.substring(0, path.lastIndexOf("/"));
      File savefile = new File(rootDir + "/" + path); //the path to the file
      File savedir = new File(rootDir + "/" + dirs); //the dir part of the path
      if(!savefile.exists())
      {//if the file isn't there create it.
        try
        {
          savedir.mkdirs(); //create any directories
        }
        catch(Exception ee)
        {
          ee.printStackTrace();
        }
      }
      
      //save a temp file so that the id can be put in the file.
      BufferedReader bfile = new BufferedReader(file);
      StringWriter sw = new StringWriter();
//      BufferedWriter bsw = new BufferedWriter(sw);
      File tempfile = new File(tempdir + "/local.noid");
      FileWriter fw = new FileWriter(tempfile);
      BufferedWriter bfw = new BufferedWriter(fw);
      int c = bfile.read();
      while(c != -1)
      {
        bfw.write(c); //write out everything in the reader
        sw.write(c);
        c = bfile.read();
      }
      bfw.flush();
      bfw.close();
      System.out.println("Starting insert");
      String fileWithId = insertIdInFile(tempfile, name); //put the id in
      if(fileWithId == null)
      {
        System.out.println("fileWithId is null!");
        fileWithId = sw.toString();
        System.out.println("fileWithId length is: "+fileWithId.length());
      }
      else {
        System.out.println("fileWithId length is: "+fileWithId.length());
        
      }
      
      //now that the id has been put in the file, we can save it.
      StringReader sr = new StringReader(fileWithId);
      BufferedReader bsr = new BufferedReader(sr);
//      while(!sr.ready())
//      {
//        int x = 1;
//      }
      BufferedWriter bwriter = new BufferedWriter(new FileWriter(savefile));
      int d = bsr.read();
      while(d != -1)
      {
        bwriter.write(d); //write out everything in the reader
        d = bsr.read();
      }
      bwriter.flush();
      bwriter.close();
      return savefile;
    }
    catch(Exception e)
    {
      e.printStackTrace();
      return null;
    }
  }
  
  /**
   * returns a File object in the local repository.
   * @param name: the id of the file
   * @param file: the stream to the file
   * @param publicAccess: flag for unauthenticated read access to the file.
   * true if anauthenticated users can read the file, false otherwise.
   */
  public File newFile(String name, Reader file, DataPackage dp)
  {
    return saveFile(name, file, datadir, dp);
  }
  
   /**
   * returns a File object in the local repository.
   * @param name: the id of the file
   * @param file: the stream to the file
   * @param publicAccess: flag for unauthenticated read access to the file.
   * true if anauthenticated users can read the file, false otherwise.
   */
  public File newFile(String name, Reader file)
  {
    return saveFile(name, file, datadir, null);
  }
  
  public File newDataFile(String name, InputStream is) {
    return saveDataFile(name, is, datadir, null);
  }
  
  /**
   * deletes a file from the local file system. returns true if the file is
   * successfully deleted, false otherwise.
   * @param name the name of the file to delete
   */
   public boolean deleteFile(String name)
   {
     String path = parseId(name);
     File delfile = new File(datadir + "/" + path); //the path to the file
     return delfile.delete();
   }
  
  /**
   * Test method
   */
  public static void main(String[] args)
  {
    String filename = args[0];
    String filename2 = args[1];
    String action = args[2];
    if(action.equals("test"))
    {
      try
      {
        ClientFramework cf = new ClientFramework(new ConfigXML("./lib/config.xml"));
        FileSystemDataStore fsds = new FileSystemDataStore(cf);
        File newfile = fsds.openFile(filename);
        fsds.saveFile(filename2, new FileReader(newfile));
      }
      catch(Exception e)
      {
        e.printStackTrace();
      }
    }
    else if(action.equals("save"))
    {
      try
      {
        ClientFramework cf = new ClientFramework(new ConfigXML("./lib/config.xml"));
        FileSystemDataStore fsds = new FileSystemDataStore(cf);
        File newfile = new File(filename);
        fsds.saveFile(filename2, new FileReader(newfile));
      }
      catch(Exception e)
      {
        e.printStackTrace();
      }
    }
    ClientFramework.debug(20, "done");
  }
  
 /**
  *  A variant of saveFile designed for use with Data Files.
  *  This avoids the writing of files to Strings that is in saveFile
  *  and allows for very large data files. (i.e. no file is put entirely
  *  in memory)
  */
  public File saveDataFile(String name, Reader file, String rootDir, DataPackage dp)
  {
    try
    {
      String path = parseId(name);
      String dirs = path.substring(0, path.lastIndexOf("/"));
      File savefile = new File(rootDir + "/" + path); //the path to the file
      File savedir = new File(rootDir + "/" + dirs); //the dir part of the path
      if(!savefile.exists())
      {//if the file isn't there create it.
        try
        {
          savedir.mkdirs(); //create any directories
        }
        catch(Exception ee)
        {
          ee.printStackTrace();
        }
      }
      
      //save a temp file so that the id can be put in the file.
      BufferedReader bfile = new BufferedReader(file);
/* 
      StringWriter sw = new StringWriter();
      BufferedWriter bsw = new BufferedWriter(sw);
      File tempfile = new File(tempdir + "/local.noid");
      FileWriter fw = new FileWriter(tempfile);
      BufferedWriter bfw = new BufferedWriter(fw);
      int c = bfile.read();
      while(c != -1)
      {
        bfw.write(c); //write out everything in the reader
        bsw.write(c);
        c = bfile.read();
      }
      bfw.flush();
      bfw.close();
      String fileWithId = insertIdInFile(tempfile, name); //put the id in
      if(fileWithId == null)
      {
        fileWithId = sw.toString();
      }
      
      //now that the id has been put in the file, we can save it.
      StringReader sr = new StringReader(fileWithId);
      BufferedReader bsr = new BufferedReader(sr);
*/      
//      while(!sr.ready())
//      {
//        int x = 1;
//      }
      BufferedWriter bwriter = new BufferedWriter(new FileWriter(savefile));
      int d = bfile.read();
      while(d != -1)
      {
        bwriter.write(d); //write out everything in the reader
        d = bfile.read();
      }
      bwriter.flush();
      bwriter.close();
      return savefile;
    }
    catch(Exception e)
    {
      e.printStackTrace();
      return null;
    }
  }

 /**
  *  A variant of saveFile designed for use with Data Files.
  *  This avoids the writing of files to Strings that is in saveFile
  *  and allows for very large data files. (i.e. no file is put entirely
  *  in memory) This version uses an InputStream rather than a Reader to
  *  avoid problems with binary file corruption
  */
  public File saveDataFile(String name, InputStream file, String rootDir, DataPackage dp)
  {
    try
    {
      String path = parseId(name);
      String dirs = path.substring(0, path.lastIndexOf("/"));
      File savefile = new File(rootDir + "/" + path); //the path to the file
      File savedir = new File(rootDir + "/" + dirs); //the dir part of the path
      if(!savefile.exists())
      {//if the file isn't there create it.
        try
        {
          savedir.mkdirs(); //create any directories
        }
        catch(Exception ee)
        {
          ee.printStackTrace();
        }
      }
      
      BufferedInputStream bfile = new BufferedInputStream(file);
      BufferedOutputStream bos = new BufferedOutputStream(new FileOutputStream(savefile));
      int d = bfile.read();
      while(d != -1)
      {
        bos.write(d); //write out everything in the reader
        d = bfile.read();
      }
      bos.flush();
      bos.close();
      bfile.close();
      return savefile;
    }
    catch(Exception e)
    {
      e.printStackTrace();
      return null;
    }
  }
  
}
